(* Copyright (c) 2011 - 2015. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: ClientDataSets"
     and
   "Delphi in Depth: ClientDataSets 2nd Edition"
   by Cary Jensen

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: ClientDataSets 2nd Edition"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/cdsbook2
*)
unit mainformu;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  FireDAC.Comp.Client,
  FireDAC.Comp.DataSet,
  FireDAC.Comp.UI,
  FireDAC.DApt,
  FireDAC.DApt.Intf,
  FireDAC.DatS,
  FireDAC.Phys,
  FireDAC.Phys.IB,
  FireDAC.Phys.IBBase,
  FireDAC.Phys.Intf,
  FireDAC.Stan.Async,
  FireDAC.Stan.Def,
  FireDAC.Stan.Error,
  FireDAC.Stan.Intf,
  FireDAC.Stan.Option,
  FireDAC.Stan.Param,
  FireDAC.Stan.Pool,
  FireDAC.UI.Intf,
  {$IF CompilerVersion < 28}
  // nothing needed
  {$ELSE}
  FireDAC.Phys.IBDef,
  {$ENDIF}
  FireDAC.VCLUI.Wait, Data.DB, Datasnap.DBClient, Datasnap.Provider,
  Vcl.StdCtrls, Vcl.Grids, Vcl.DBGrids, Vcl.DBCtrls, Vcl.ExtCtrls, Vcl.ComCtrls;


type
  TForm1 = class(TForm)
    FDConnection1: TFDConnection;
    FDQuery1: TFDQuery;
    FDGUIxWaitCursor1: TFDGUIxWaitCursor;
    FDPhysIBDriverLink1: TFDPhysIBDriverLink;
    Panel1: TPanel;
    Panel2: TPanel;
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    Label1: TLabel;
    FilterExpressionEdit: TEdit;
    ApplyFilterExpressionButton: TButton;
    DataSetProvider1: TDataSetProvider;
    ClientDataSet1: TClientDataSet;
    DataSource1: TDataSource;
    Button10: TButton;
    Button11: TButton;
    Button12: TButton;
    Button13: TButton;
    Button14: TButton;
    Button15: TButton;
    Button16: TButton;
    Button17: TButton;
    Button18: TButton;
    Button19: TButton;
    Button20: TButton;
    Button21: TButton;
    Button22: TButton;
    Button25: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    Button7: TButton;
    Button8: TButton;
    Button9: TButton;
    Button26: TButton;
    Button27: TButton;
    Button28: TButton;
    Button29: TButton;
    Button30: TButton;
    Button31: TButton;
    Button32: TButton;
    Button33: TButton;
    ClientDataSet1CUSTNO: TFloatField;
    ClientDataSet1COMPANY: TStringField;
    ClientDataSet1ADDR1: TStringField;
    ClientDataSet1ADDR2: TStringField;
    ClientDataSet1CITY: TStringField;
    ClientDataSet1STATE: TStringField;
    ClientDataSet1ZIP: TStringField;
    ClientDataSet1COUNTRY: TStringField;
    ClientDataSet1PHONE: TStringField;
    ClientDataSet1FAX: TStringField;
    ClientDataSet1TAXRATE: TFloatField;
    ClientDataSet1CONTACT: TStringField;
    ClientDataSet1LASTINVOICEDATE: TSQLTimeStampField;
    ClientDataSet1LASTINVDATE: TDateField;
    ClientDataSet1LASTINVTIME: TTimeField;
    Label2: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Button23: TButton;
    StatusBar1: TStatusBar;
    CaseInsensitive: TCheckBox;
    NoPartialCompare: TCheckBox;
    Label3: TLabel;
    Label6: TLabel;
    procedure AssignFilter(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ClearFilter(Sender: TObject);
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure CaseInsensitiveClick(Sender: TObject);
    procedure NoPartialCompareClick(Sender: TObject);
    procedure ApplyFilterExpressionButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.AssignFilter(Sender: TObject);
begin
  ClientDataSet1.Filter :=  TButton(Sender).Caption;
end;

procedure TForm1.ApplyFilterExpressionButtonClick(Sender: TObject);
begin
  ClientDataSet1.Filter := FilterExpressionEdit.Text;
end;

procedure TForm1.CaseInsensitiveClick(Sender: TObject);
begin
  if CaseInsensitive.Checked then
    ClientDataSet1.FilterOptions := ClientDataSet1.FilterOptions +
                                      [foCaseInsensitive]
  else
    ClientDataSet1.FilterOptions := ClientDataSet1.FilterOptions -
                                      [foCaseInsensitive];
end;

procedure TForm1.ClearFilter(Sender: TObject);
begin
  ClientDataSet1.Filter := '';
end;

procedure TForm1.DataSource1DataChange(Sender: TObject; Field: TField);
begin
  StatusBar1.SimpleText := 'Record in view: ' + ClientDataSet1.RecordCount.ToString
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  DataDir: string;
begin
  DataDir := GetEnvironmentVariable('DEMOSDIR');
  if DataDir.IsEmpty then
  begin
    ShowMessage('DEMOSDIR environment variable is not set. ' +
                'Cannot continue. See Appendix A, Setting the ' +
                'DEMOSDIR environment variable for more information');
    exit;
  end;

  FDConnection1.Params.Clear;
  FDConnection1.Params.Add('DriverID=IB');
  FDConnection1.Params.Add('Database=' +
    GetEnvironmentVariable('DEMOSDIR') + 'Data\dbdemos.gdb');
  FDConnection1.Params.Add('User_Name=sysdba');
  FDConnection1.Params.Add('password=masterkey');
  FDConnection1.Params.Add('Server=localhost');
  FDCOnnection1.LoginPrompt := False;

  ClientDataSet1.Filter := '';
  ClientDataSet1.Filtered := True;
  ClientDataSet1.Open;
end;

procedure TForm1.NoPartialCompareClick(Sender: TObject);
begin
  if NoPartialCompare.Checked then
    ClientDataSet1.FilterOptions := ClientDataSet1.FilterOptions +
                                      [foNoPartialCompare]
  else
    ClientDataSet1.FilterOptions := ClientDataSet1.FilterOptions -
                                      [foNoPartialCompare];

end;

end.
