(* Copyright (c) 2011 - 2015. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: ClientDataSets"
     and
   "Delphi in Depth: ClientDataSets 2nd Edition"
   by Cary Jensen

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: ClientDataSets 2nd Edition"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/cdsbook2
*)
unit mainformu;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, StdCtrls, DBCtrls, Grids, DBGrids, ExtCtrls, DBClient,
  Provider,
  FireDAC.Comp.Client,
  FireDAC.Comp.DataSet,
  FireDAC.Comp.UI,
  FireDAC.DApt,
  FireDAC.DApt.Intf,
  FireDAC.DatS,
  FireDAC.Phys,
  FireDAC.Phys.IB,
  FireDAC.Phys.IBBase,
  FireDAC.Phys.Intf,
  FireDAC.Stan.Async,
  FireDAC.Stan.Def,
  FireDAC.Stan.Error,
  FireDAC.Stan.Intf,
  FireDAC.Stan.Option,
  FireDAC.Stan.Param,
  FireDAC.Stan.Pool,
  FireDAC.UI.Intf,
  {$IF CompilerVersion < 28}
  // nothing needed
  {$ELSE}
  FireDAC.Phys.IBDef,
  {$ENDIF}
  FireDAC.VCLUI.Wait;

type
  TForm1 = class(TForm)
    DataSource1: TDataSource;
    Panel2: TPanel;
    DBGrid1: TDBGrid;
    Label1: TLabel;
    Splitter1: TSplitter;
    Panel3: TPanel;
    Label2: TLabel;
    DBGrid2: TDBGrid;
    DataSource2: TDataSource;
    DataSetProvider1: TDataSetProvider;
    ClientDataSet1: TClientDataSet;
    Panel1: TPanel;
    DBNavigator1: TDBNavigator;
    Button2: TButton;
    FDGUIxWaitCursor1: TFDGUIxWaitCursor;
    FDQuery1: TFDQuery;
    FDConnection1: TFDConnection;
    FDQuery2: TFDQuery;
    FDTable1: TFDTable;
    procedure FormCreate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure DataSetProvider1BeforeUpdateRecord(Sender: TObject;
      SourceDS: TDataSet; DeltaDS: TCustomClientDataSet;
      UpdateKind: TUpdateKind; var Applied: Boolean);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    procedure PrepareTempTable(Conn: TFDConnection);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
var
  DataDir: string;
begin
  DataDir := GetEnvironmentVariable('DEMOSDIR');
  if DataDir.IsEmpty then
  begin
    ShowMessage('DEMOSDIR environment variable is not set. ' +
                'Cannot continue. See Appendix A, Setting the ' +
                'DEMOSDIR environment variable for more information');
    exit;
  end;

  FDConnection1.Params.Clear;
  FDConnection1.Params.Add('DriverID=IB');
  FDConnection1.Params.Add('Database=' +
    GetEnvironmentVariable('DEMOSDIR') + 'Data\dbdemos.gdb');
  FDConnection1.Params.Add('User_Name=sysdba');
  FDConnection1.Params.Add('password=masterkey');
  FDConnection1.Params.Add('Server=localhost');
  FDCOnnection1.LoginPrompt := False;

  PrepareTempTable(FDConnection1);


  FDQuery1.Connection := FDConnection1;
  FDTable1.Connection := FDConnection1;
  FDQuery2.Connection := FDConnection1;
  FDTable1.Open;
  ClientDataSet1.Open;
  FDQuery2.Prepare;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
if ClientDataSet1.State in [dsInsert, dsEdit] then
  ClientDataSet1.Post;
if ClientDataSet1.ChangeCount = 0 then
  begin
    ShowMessage('No changes to apply');
    Exit;
  end;
ClientDataSet1.ApplyUpdates(-1);
FDTable1.Refresh;
end;

procedure TForm1.DataSetProvider1BeforeUpdateRecord(Sender: TObject;
  SourceDS: TDataSet; DeltaDS: TCustomClientDataSet;
  UpdateKind: TUpdateKind; var Applied: Boolean);
begin
  Applied := True;
  if UpdateKind = ukModify then
  begin
    FDQuery2.ParamByName('PartNo').Value :=
      DeltaDS.FieldByName('PartNo').NewValue;
    FDQuery2.ParamByName('OLD_PartNo').Value :=
      DeltaDS.FieldByName('PartNo').OldValue;
    FDQuery2.ExecSQL;
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FDQuery2.UnPrepare;
  FDConnection1.ExecSQL('DROP TABLE DemoTable');
end;

procedure TForm1.PrepareTempTable(Conn: TFDConnection);
var
  Query: TFDQuery;
begin
  Query := TFDQuery.Create(nil);
  try
    Query.Connection := Conn;
    Query.SQL.Text := 'SELECT rdb$relation_name ' +
                      '  FROM rdb$relations ' +
                      '  WHERE rdb$relation_name = ''DEMOTABLE'' ';
    Query.Open();
    if not Query.IsEmpty then
      Conn.ExecSQL('DROP TABLE DemoTable');
    Conn.ExecSQL('CREATE TABLE DemoTable ' +
                 '  (ORDERNO DOUBLE PRECISION, ' +
                 '   ITEMNO DOUBLE PRECISION,'+
                 '   PARTNO DOUBLE PRECISION, ' +
                 '   QTY INTEGER, ' +
                 '   DISCOUNT DOUBLE PRECISION)');
    Conn.ExecSQL('INSERT INTO DemoTable SELECT * FROM ITEMS');
  finally
    Query.Free;
  end;
end;

end.
