(* Copyright (c) 2011 - 2015. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: ClientDataSets"
     and
   "Delphi in Depth: ClientDataSets 2nd Edition"
   by Cary Jensen

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: ClientDataSets 2nd Edition"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/cdsbook2
*)
unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Provider, DB, DBClient, ExtCtrls, Grids,
  DBGrids, DBCtrls, Menus, ComCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    DBNavigator2: TDBNavigator;
    DBGrid2: TDBGrid;
    DataSource1: TDataSource;
    ClientDataSet1: TClientDataSet;
    Panel2: TPanel;
    IndexListBox: TListBox;
    Label1: TLabel;
    Edit1: TEdit;
    Edit2: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    DeleteRangeButton: TButton;
    CancelUpdatesButton: TButton;
    OpenDialog1: TOpenDialog;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    Close1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    VisitwwwDephiDevelperDayscom1: TMenuItem;
    VisitwwwJensenDataSystemscom1: TMenuItem;
    N2: TMenuItem;
    About1: TMenuItem;
    StatusBar1: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure VisitwwwDephiDevelperDayscom1Click(Sender: TObject);
    procedure VisitwwwJensenDataSystemscom1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure DeleteRangeButtonClick(Sender: TObject);
    procedure CancelUpdatesButtonClick(Sender: TObject);
    procedure ClientDataSet1AfterOpen(DataSet: TDataSet);
    procedure Open1Click(Sender: TObject);
    procedure ClientDataSet1AfterClose(DataSet: TDataSet);
    procedure Close1Click(Sender: TObject);
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

function CDSDeleteRange(SourceCDS: TClientDataSet; const IndexFieldNames: String;
  const StartValues, EndValues: array of const): Integer;

implementation

{$R *.dfm}

uses ShellApi;

function CDSDeleteRange(SourceCDS: TClientDataSet;
  const IndexFieldNames: String;
  const StartValues, EndValues: array of const): Integer;
var
  Clone: TClientDataSet;
begin
  //initialize number of deleted records
  Result := 0;
  Clone := TClientDataSet.Create(nil);
  SourceCDS.DisableControls;
  try
    Clone.CloneCursor(SourceCDS, True);
    Clone.IndexFieldNames := IndexFieldNames;
    Clone.SetRange(StartValues, EndValues);
    while Clone.RecordCount > 0 do
      begin
        Clone.Delete;
        Inc(Result);
      end;
  finally
    SourceCDS.EnableControls;
    Clone.Free;
  end;
end;

procedure TForm1.ClientDataSet1AfterClose(DataSet: TDataSet);
begin
Close1.Enabled := False;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  ClientDataSet1.FileName := ExtractFilePath(Application.ExeName) +
    '\..\bigcds\bigcds.cds';
  if not FileExists(ClientDataSet1.FileName) then
  begin
    ShowMessage('BigCDS.cds not found. This project requires this ' +
      'file, and expects to find it in the folder named BigCDS, ' +
      'which should be located in the folder directly above ' +
      ExtractFilePath(Application.ExeName) + '. Cannot continue.');
    Abort;
  end;
  ClientDataSet1.Open;
end;

procedure TForm1.DeleteRangeButtonClick(Sender: TObject);
begin
if (Edit1.Text = '') and (Edit2.Text = '') then
  begin
    ShowMessage('Enter a range before attempting to delete');
    Exit;
  end;
  Screen.Cursor := crHourGlass;
  try
    ShowMessage(IntToStr(CDSDeleteRange(ClientDataSet1,
      IndexListBox.Items[IndexListBox.ItemIndex],
      [Edit1.Text],[Edit2.Text])) + ' records were deleted');
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.CancelUpdatesButtonClick(Sender: TObject);
begin
  ClientDataSet1.DisableControls;
  try
    ClientDataSet1.CancelUpdates;
  finally
    ClientDataSet1.EnableControls;
  end;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
Exit;
end;

procedure TForm1.VisitwwwDephiDevelperDayscom1Click(Sender: TObject);
begin
ShellExecute(Application.Handle, 'open',
 'http://www.DelphiDeveloperDays.com', nil, nil, SW_SHOW);
end;

procedure TForm1.VisitwwwJensenDataSystemscom1Click(Sender: TObject);
begin
 ShellExecute(Application.Handle, 'open',
 'http://www.JensenDataSystems.com', nil, nil, SW_SHOW);
end;

procedure TForm1.About1Click(Sender: TObject);
begin
ShowMessage('CDSDeleteRange (c) 2002 Cary Jensen, Jensen Data Systems, Inc. '
  + #10#13
  + #10#13 + 'Demonstration of how to use a cloned ClientDataSet'
  + #10#13 + 'to implement features. In this case, the clone is used '
  + #10#13 + 'to delete a range of records from a ClientDataSet without '
  + #10#13 + 'affecting that ClientDataSet''s index.'
  + #10#13
  + #10#13 + 'This project is provided for demonstration purposes only.'
  + #10#13
  + #10#13 + 'Visit http://www.JensenDataSystems.com for information about training and consulting services.');

end;

procedure TForm1.ClientDataSet1AfterOpen(DataSet: TDataSet);
var
  i: Integer;
begin
for i := 0 to Pred(ClientDataSet1.Fields.Count) do
  IndexListBox.Items.Add(ClientDataSet1.Fields[i].FieldName);
IndexListBox.ItemIndex := 0;
Close1.Enabled := True;
end;

procedure TForm1.Open1Click(Sender: TObject);
begin
if OpenDialog1.Execute then
begin
  ClientDataSet1.Close;
  ClientDataSet1.FileName := OpenDialog1.FileName;
  ClientDataSet1.Open;
end;
end;

procedure TForm1.Close1Click(Sender: TObject);
begin
ClientDataSet1.Close;
IndexListBox.Items.Clear;
end;

procedure TForm1.DataSource1DataChange(Sender: TObject; Field: TField);
begin
  StatusBar1.SimpleText := IntToStr(ClientDataSet1.RecNo) +
    ' of ' + IntToStr(ClientDataSet1.RecordCount) + ' records';
end;

end.
