(* Copyright (c) 2011 - 2015. Cary Jensen, Jensen Data Systems, Inc.
   This code sample is intended for the readers of
   "Delphi in Depth: ClientDataSets"
     and
   "Delphi in Depth: ClientDataSets 2nd Edition"
   by Cary Jensen

   No guarantees or warranties are expressed or implied concerning
   the applicability of techniques or code included in this example
   or in the accompanying book. If you wish to use techniques or
   code included in this example or described in the book, it is
   your responsibility to test and certify any code, techniques,
   or design you adopt.

   For information on consulting or training services, please visit:
   http://www.JensenDataSystems.com

   For more information about "Delphi in Depth: ClientDataSets 2nd Edition"
   by Cary Jensen, including links to order the book, please visit:
   http://www.JensenDataSystems.com/cdsbook2
*)
unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, DB, DBClient, Grids, DBGrids,
  DBCtrls, Menus, Provider,
  FireDAC.Comp.Client,
  FireDAC.Comp.DataSet,
  FireDAC.Comp.UI,
  FireDAC.DApt,
  FireDAC.DApt.Intf,
  FireDAC.DatS,
  FireDAC.Phys,
  FireDAC.Phys.IB,
  FireDAC.Phys.IBBase,
  FireDAC.Phys.Intf,
  FireDAC.Stan.Async,
  FireDAC.Stan.Def,
  FireDAC.Stan.Error,
  FireDAC.Stan.Intf,
  FireDAC.Stan.Option,
  FireDAC.Stan.Param,
  FireDAC.Stan.Pool,
  FireDAC.UI.Intf,
  {$IF CompilerVersion < 28}
  // nothing needed
  {$ELSE}
  FireDAC.Phys.IBDef,
  {$ENDIF}
  FireDAC.VCLUI.Wait;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Splitter1: TSplitter;
    Panel3: TPanel;
    Splitter2: TSplitter;
    Panel4: TPanel;
    Splitter3: TSplitter;
    Panel5: TPanel;
    StatusBar1: TStatusBar;
    FirstBtn: TButton;
    NextBtn: TButton;
    PriorBtn: TButton;
    LastBtn: TButton;
    ScanForwardBtn: TButton;
    ScanBackwardBtn: TButton;
    ControlsStateBtnGrp: TRadioGroup;
    UpDown1: TUpDown;
    Edit1: TEdit;
    MoveByBtn: TButton;
    Panel6: TPanel;
    DBNavigator1: TDBNavigator;
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    ClientDataSet1: TClientDataSet;
    OpenDialog1: TOpenDialog;
    Edit2: TEdit;
    UpDown2: TUpDown;
    RecNoBtn: TButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Close1: TMenuItem;
    DataSetProvider1: TDataSetProvider;
    FDGUIxWaitCursor1: TFDGUIxWaitCursor;
    FDQuery1: TFDQuery;
    FDConnection1: TFDConnection;
    procedure FormCreate(Sender: TObject);
    procedure DataSource1DataChange(Sender: TObject; Field: TField);
    procedure DataSource1StateChange(Sender: TObject);
    procedure FirstBtnClick(Sender: TObject);
    procedure NextBtnClick(Sender: TObject);
    procedure PriorBtnClick(Sender: TObject);
    procedure LastBtnClick(Sender: TObject);
    procedure ScanForwardBtnClick(Sender: TObject);
    procedure ScanBackwardBtnClick(Sender: TObject);
    procedure MoveByBtnClick(Sender: TObject);
    procedure RecNoBtnClick(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure ClientDataSet1AfterClose(DataSet: TDataSet);
    procedure ClientDataSet1AfterOpen(DataSet: TDataSet);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses typinfo;


procedure TForm1.FormCreate(Sender: TObject);
var
  DataDir: string;
begin
  DataDir := GetEnvironmentVariable('DEMOSDIR');
  if DataDir.IsEmpty then
  begin
    ShowMessage('DEMOSDIR environment variable is not set. ' +
                'Cannot continue. See Appendix A, Setting the ' +
                'DEMOSDIR environment variable for more information');
    exit;
  end;

  FDConnection1.Params.Clear;
  FDConnection1.Params.Add('DriverID=IB');
  FDConnection1.Params.Add('Database=' +
    GetEnvironmentVariable('DEMOSDIR') + 'Data\dbdemos.gdb');
  FDConnection1.Params.Add('User_Name=sysdba');
  FDConnection1.Params.Add('password=masterkey');
  FDConnection1.Params.Add('Server=localhost');
  FDCOnnection1.LoginPrompt := False;

  ClientDataSet1.Open;
end;

procedure TForm1.DataSource1DataChange(Sender: TObject; Field: TField);
begin
StatusBar1.Panels[0].Text := 'Record ' +
  IntToStr(ClientDataSet1.RecNo) + ' of ' +
  IntToStr(ClientDataSet1.RecordCount);
StatusBar1.Panels[2].Text :=
  'BOF = ' + BoolToStr(ClientDataSet1.Bof, True) + '. ' +
  'EOF = ' + BoolToStr(ClientDataSet1.Eof, True) + '. ';
end;

procedure TForm1.DataSource1StateChange(Sender: TObject);
begin
StatusBar1.Panels[1].Text :=
  'State = ' + GetEnumName(TypeInfo(TDataSetState),
  Ord(ClientDataSet1.State));
end;

procedure TForm1.FirstBtnClick(Sender: TObject);
begin
ClientDataSet1.First;
end;

procedure TForm1.NextBtnClick(Sender: TObject);
begin
ClientDataSet1.Next;
end;

procedure TForm1.Open1Click(Sender: TObject);
begin
  ClientDataSet1.Open;
end;

procedure TForm1.PriorBtnClick(Sender: TObject);
begin
ClientDataSet1.Prior;
end;

procedure TForm1.LastBtnClick(Sender: TObject);
begin
ClientDataSet1.Last;
end;

procedure TForm1.ScanForwardBtnClick(Sender: TObject);
begin
if ControlsStateBtnGrp.ItemIndex = 1 then
  ClientDataSet1.DisableControls;
  try
    ClientDataSet1.First;
    while not ClientDataSet1.Eof do
    begin
      //do something with a record
      ClientDataSet1.Next;
    end;
  finally
    if ControlsStateBtnGrp.ItemIndex = 1 then
      ClientDataSet1.EnableControls;
  end;
end;

procedure TForm1.ScanBackwardBtnClick(Sender: TObject);
begin
if ControlsStateBtnGrp.ItemIndex = 1 then
  ClientDataSet1.DisableControls;
  try
    ClientDataSet1.Last;
    while not ClientDataSet1.Bof do
    begin
      //do something with a record
      ClientDataSet1.Prior;
    end;
  finally
    if ControlsStateBtnGrp.ItemIndex = 1 then
      ClientDataSet1.EnableControls;
  end;
end;

procedure TForm1.MoveByBtnClick(Sender: TObject);
begin
ClientDataSet1.MoveBy(UpDown1.Position);
end;

procedure TForm1.RecNoBtnClick(Sender: TObject);
begin
  ClientDataSet1.RecNo := UpDown2.Position;
end;

procedure TForm1.ClientDataSet1AfterClose(DataSet: TDataSet);
begin
  Open1.Enabled := True;
  Close1.Enabled := False;
end;

procedure TForm1.ClientDataSet1AfterOpen(DataSet: TDataSet);
begin
  Open1.Enabled := False;
  Close1.Enabled := True;
end;

procedure TForm1.Close1Click(Sender: TObject);
begin
  ClientDataSet1.Close;
end;

end.
